/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * GameManager.cpp - manages all the game-related stuff
 *
 ************************************************************************/

#include <cassert>

#include "GameManager.h"
#include "GridDrawer.h"
#include "UserInterface.h"
#include "GameGrid.h"
#include "ScoreController.h"

#ifdef USE_CONSOLE
	#include "TextUserInterface.h"
#else
	#include "SdlUserInterface.h"
#endif


const short N_ROWS = 5;
const short N_SLOTS = 7;



/*
 * Ctor...
 */
GameManager::GameManager()
	: isRunning_(false),
	  doQuit_(false),
	  winner_(NO_PLAYER),
	  actualPlayer_(0)
{
	typedef std::pair<PlayerCode, Player> PlayerPair;
	playerMap_[PLAYER_1] = Player("Player 1", PLAYER_1, 255, 064, 064, 'O');
	playerMap_[PLAYER_2] = Player("Player 2", PLAYER_2, 128, 128, 255, 'X');

	// the following is not a player per se, but if there's no NO_WINNER
	// in this map, then we'll have a problem if the game ends in a draw
	// and we have call UserInterface::showWinner(playerMap_[NO_WINNER])
	playerMap_[NO_WINNER] = Player("NO_WINNER", NO_WINNER, 0, 0, 0, ' ');

	grid_ = new GameGrid(N_ROWS, N_SLOTS);
	scoreController_ = new ScoreController(*grid_);
#ifdef USE_CONSOLE
	userInterface_ = new TextUserInterface(*this);
#else
	userInterface_ = new SdlUserInterface(*this);
#endif
}



/*
 * Dtor
 */
GameManager::~GameManager()
{
	delete grid_;
	delete scoreController_;
	delete userInterface_;
}

/*
 * this starts the game :)
 */
void GameManager::start()
{
	while( !doQuit_)
	{
		userInterface_->showMainMenu();
	}
}


/*
 * run the game
 * this is the actual game-loop
 */
void GameManager::run()
{
	isRunning_ = true;
	actualPlayer_ = &(playerMap_[PLAYER_1]);
	grid_->reset();
	while (isRunning_)
	{
		userInterface_->handleInput();
		winner_ = scoreController_->checkForWinner();

		if (winner_ != NO_PLAYER)
		{
			isRunning_ = false;
			userInterface_->showWinner(playerMap_[winner_]);
		}
	}
}



/*
 * ends a game and returns to the main-menu
 */
void GameManager::exitToMainMenu()
{
	isRunning_ = false;
}



/*
 * ends a game and return to the OS
 */
void GameManager::exitProgram()
{
	isRunning_ = false;
	doQuit_ = true;
}


/*
 * put a coin into a slot... if the slot-index is illegal or the slot
 * is full, an "IllegalSlotError" exception will be thrown
 */
void GameManager::putIntoSlot(unsigned short slot)
{
	if (slot >= grid_->getSlots())
		throw IllegalSlotError();

	if (!grid_->putIntoSlot(slot, getActualPlayer().code))
		throw IllegalSlotError();
	else
		switchActualPlayer();
}




/*
 * changes the actual player
 */
void GameManager::switchActualPlayer()
{
	if (actualPlayer_->code == PLAYER_1)
		actualPlayer_ = &(playerMap_[PLAYER_2]);
	else
		actualPlayer_ = &(playerMap_[PLAYER_1]);
}
