/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * SdlFontManager.cpp - a simple wrapper over SDL_ttf
 *
 ************************************************************************/

#include "SdlFontManager.h"

#include <sstream>
using std::ostringstream;
#include <SDL.h>
#include <SDL/SDL_ttf.h>
#include "SdlHelper.h"

/*
 * private Ctor
 */
SdlFontManager::SdlFontManager()
	: quitFontSubsystemOnExit_(false)
{
	if (SDL_WasInit(SDL_INIT_VIDEO) ==0)
		throw std::runtime_error("cannot initialize FontSystem if SDL is not initialized\n");

	if (!TTF_WasInit())
	{
		if (TTF_Init() != 0)
		{
			throw std::runtime_error("failed to initialize SDL_ttf");
		}
		quitFontSubsystemOnExit_ = true;
	}
}


/*
 * private Dtor.
 * If necessary delete the SDL_ttf subsystem.
 */
SdlFontManager::~SdlFontManager()
{
	for (FontMap::iterator it = fonts_.begin(); it != fonts_.end(); ++it)
	{
		TTF_CloseFont(it->second->font_);
		delete it->second;
	}
	if (quitFontSubsystemOnExit_)
		TTF_Quit();
}



/*
 * implements the singleton
 */
SdlFontManager& SdlFontManager::get()
{
	static SdlFontManager manager;
	return manager;
}



/*
 * return the requested font.
 * If the font wasn't loaded already, load it now.
 */
SdlFont& SdlFontManager::getFont(const string name, int size)
{
	string request = makeKey(name, size);
	FontMap::const_iterator it = fonts_.find(request);
	if (it != fonts_.end())
	{
		return *(it->second);
	}
	else
	{
		TTF_Font* newFont = TTF_OpenFont(getPath(name).c_str(), size);
		if (!newFont)
			throw std::runtime_error("SdlFontManager: unable to load " + name);

		fonts_[request] = new SdlFont(newFont);
		SdlFont& retval = *(fonts_[request]);
		return retval;
	}
}



/*
 * converts name and size to a string-only object.
 * That object can than be used as key in the fonts_ - map
 */
string SdlFontManager::makeKey(const string& name, int size)
{
	ostringstream str;
	str << name << size;
	return str.str();
}

