/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * SdlFontHelper.cpp - some helper functions for libSDL
 *
 ************************************************************************/


#include "SdlHelper.h"
#include <SDL.h>
#include <cassert>


/*
 * loads an Image from File, and if necessary sets the colorkey to (r, g, b)
 * The loaded file will be converted to displayformat.
 * NOTE: only BMP-format is supported at the moment
 * NOTE: setColorKey defaults to false, the Colorkey defaults to (255, 0, 255)
 * NOTE: only provide filename, NOT its location (path)
 */
SDL_Surface* loadImage(const string& name, bool setColorKey, Uint8 r, Uint8 g, Uint8 b)
{
	SDL_Surface* tmp  = SDL_LoadBMP(getPath(name).c_str());
	assert(tmp);
	SDL_Surface* retval = 0;
	if (setColorKey)
	{
		SDL_SetColorKey(tmp, SDL_SRCCOLORKEY | SDL_RLEACCEL,
			SDL_MapRGB(tmp->format, r, g, b));
	}
	retval = SDL_DisplayFormat(tmp);
	assert(retval);
	SDL_FreeSurface(tmp);
	return retval;
}


/*
 * creates a surface for renderMultiline*() - methods.
 * The surface has to be big enough to contain all the surfaces
 * created by the renderMultiline() methods.
 * r, g, b == the background color (or the transparent key, setColorKey is true)
 */
SDL_Surface* createMultilineReturnSurface(int width, int height,
                                          Uint8 r, Uint8 g, Uint8 b,
                                          bool setColorKey)
{
	SDL_Surface* display = SDL_GetVideoSurface();
	SDL_Surface* retval = SDL_CreateRGBSurface(SDL_HWSURFACE,
		width, height, display->format->BitsPerPixel, display->format->Rmask,
		display->format->Gmask, display->format->Bmask, display->format->Amask);
	assert(display && retval);

	Uint32 color = SDL_MapRGB(retval->format, r, g, b);
	SDL_FillRect(retval, 0, color);
	if (setColorKey)
		SDL_SetColorKey(retval, SDL_SRCCOLORKEY | SDL_RLEACCEL, color);

	return retval;
}



/*
 * blits multiple surfaces on one single surface
 * r, g, b: the backgroundcolor or colorkey (if makeRestTransparent is true)
 */
SDL_Surface* blitOnOneSurface(vector<SDL_Surface*> surfaces, Alignment align,
                              Uint8 r, Uint8 g, Uint8 b, bool setColorKey,
                             int spaceBetweenSurfaces)
{
	int finalHeight = 0;
	int finalWidth = 0;

	// create the single surfaces
	for (int i = 0; i < surfaces.size(); ++i)
	{
		finalHeight += surfaces[i]->h + spaceBetweenSurfaces;
		if (surfaces[i]->w > finalWidth) finalWidth = surfaces[i]->w;
	}

	SDL_Surface* retval =
		createMultilineReturnSurface(finalWidth, finalHeight, r, g, b, setColorKey);

	// blit on the big surface
	SDL_Rect blitRect;
	blitRect.x = blitRect.y = 0;
	for (int i = 0; i < surfaces.size(); ++i)
	{
		switch (align)
		{
			case ALIGN_LEFT:   blitRect.x = 0; break;
			case ALIGN_RIGHT:  blitRect.x = finalWidth - surfaces[i]->w; break;
			case ALIGN_CENTER: blitRect.x = (finalWidth - surfaces[i]->w) / 2; break;
		}
		blitRect.h = surfaces[i]->h;
		SDL_BlitSurface(surfaces[i], 0, retval, &blitRect);
		blitRect.y += surfaces[i]->h + spaceBetweenSurfaces;
	}

	return retval;
}

