/************************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * SdlMenu.cpp - implements a menu using libSDL
 *
 ************************************************************************/


#include "SdlMenu.h"
#include "SdlFontManager.h"
#include <SDL.h>

#include <cassert>

const int SPACE_BETWEEN_ENTRIES = 10;



/*
 * Ctor
 */
SdlMenu:: SdlMenu(SDL_Surface* surface, int abortId)
	: surface_(surface),
	  abortId_(abortId)
{
	isDisplaySurface = surface == SDL_GetVideoSurface();
	selectedFont_ = &(SdlFontManager::get().getFont("Efont_Serif.ttf", 72));
	unselectedFont_ = &(SdlFontManager::get().getFont("Efont_Serif.ttf", 36));
	aboutText_ = makeAboutText();
}


/*
 * Dtor
 */
SdlMenu::~SdlMenu()
{
	for (EntryIter i = entries_.begin(); i != entries_.end(); ++i)
	{
		SDL_FreeSurface(i->unselectedSurface);
		SDL_FreeSurface(i->selectedSurface);
	}
	SDL_FreeSurface(aboutText_);
}


/*
 * adds a new selection, creating the two surfaces needed
 * to display it later on: one if the entry is selected
 * and one for when it's not
 *
 * the surfaces created here will later be deleted within
 * the Dtor
 */
void SdlMenu::addEntry(const string& label, int return_id)
{
	SDL_Color green = {0, 255, 0};
	SDL_Surface* selected = selectedFont_->renderBlended(label, 0, 255, 0);
	SDL_Surface* unselected = unselectedFont_->renderBlended(label, 128, 128, 128);

	entries_.push_back(Entry (label, return_id, selected, unselected));
}


/*
 * Message-Loop running as long as no entry gets selected.
 * NOTE:
 * drawing on the screen will only be done when we recieve a new Event.
 * that way we don't have too much to draw and save CPU-Power :)
 *
 */
int SdlMenu::run()
{
	SDL_ShowCursor(SDL_ENABLE);
	SDL_Event event;
	selectedEntry_ = entries_.begin();

	// note that we clear all pending events now just to make sure
	// we don't deal with events that were released before the menu
	// actually was active.
	while (SDL_PollEvent(0)) SDL_PollEvent(&event);

	// draw everything now, since later on we'll draw only when
	// we recieve a new Event.
	drawMenu();

	while (true) // abort loop only within return-statements
	{
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
				case SDL_KEYUP:
					switch(event.key.keysym.sym)
					{
						case SDLK_UP:
							if (selectedEntry_ == entries_.begin())
							    selectedEntry_ = entries_.end();
							--selectedEntry_;
							break;
						case SDLK_DOWN:
							++selectedEntry_;
							if (selectedEntry_ == entries_.end())
							    selectedEntry_ = entries_.begin();
							break;
						case SDLK_RETURN:	return selectedEntry_->id;
						case SDLK_ESCAPE:	return abortId_;
					}
					break;

				case SDL_MOUSEBUTTONUP:
					if (event.button.button == SDL_BUTTON_LEFT)
					{
						if (selectElementUnderMouse(event.button.x, event.button.y))
						    return selectedEntry_->id;
					}
					break;
					
				case SDL_MOUSEMOTION:
					selectElementUnderMouse(event.button.x, event.button.y);
					break;

				case SDL_QUIT:
					// we simpy jump out of this function and let
					// the caller deal with this message :)
					SDL_PushEvent(&event);
					return abortId_;
			}
			drawMenu();
		}
		SDL_Delay(10); // don't eat to much CPU-Power ;-)
	}
}



/*
 * draws the entries centered on the surface
 */
void SdlMenu::drawEntries()
{
	if (entries_.size() == 0) return;

	int height_needed = (entries_[0].selectedSurface->h + SPACE_BETWEEN_ENTRIES) * entries_.size();
	assert(height_needed < surface_->h);

	SDL_Rect rect;
	SDL_Surface* tmpSurface;
	rect.y = surface_->h / 2 - height_needed / 2; // y - start
	for (EntryIter entry = entries_.begin(); entry != entries_.end(); ++entry)
	{
		if (entry == selectedEntry_)
			tmpSurface = entry->selectedSurface;
		else
			tmpSurface = entry->unselectedSurface;

		rect.x = surface_-> w / 2 - tmpSurface->w / 2;
		SDL_BlitSurface(tmpSurface, 0, surface_, &rect);
		rect.y += tmpSurface->h + SPACE_BETWEEN_ENTRIES;
	}
}




/*
 * redraw the background of surface_.
 * TODO: for now this just blanks the screen
 */
void SdlMenu::drawBackground()
{
	static Uint32 fillcolor = SDL_MapRGB(SDL_GetVideoSurface()->format, 255, 255, 255);
	static SDL_Rect aboutTextPosition;
	aboutTextPosition.x = surface_->w - aboutText_->w;
	aboutTextPosition.y = surface_->h - aboutText_->h;
	SDL_FillRect(surface_, 0, fillcolor);
	SDL_BlitSurface(aboutText_, 0, surface_, &aboutTextPosition);
}



/*
 * create the about-text.
 * as SDL_ttf can't render a multiline text, we'll construct
 * such a text by rendering the text line per line and then
 * blitting the whole thing on one big surface :)
 */
SDL_Surface* SdlMenu::makeAboutText()
{
	SdlFont font = SdlFontManager::get().getFont("Efont_Serif.ttf", 14);
	SDL_Surface* retval = font.renderMultilineBlended(
		"-- VierVerliert --\n(c) 2004, 2005\nThomas Unterthiner",
		0, 0, 255, ALIGN_CENTER);

	return retval;
}



/*
 * check if a mouseclick taking place on given (x,y) coordinates
 * has hit a menuentry.
 * If it has, we change the selectedEntry_ Iterator and return true.
 * Otherwhise we return false.
 */
bool SdlMenu::selectElementUnderMouse(int x, int y)
{
	if (entries_.size() == 0) return false;
	int height_needed = (entries_[0].selectedSurface->h + SPACE_BETWEEN_ENTRIES) * entries_.size();
	assert(height_needed < surface_->h);

	SDL_Rect rect;
	SDL_Surface* tmpSurface = 0;
	rect.y = surface_->h / 2 - height_needed / 2; // y - start

	for (EntryIter entry = entries_.begin(); entry != entries_.end(); ++entry)
	{
		if (entry == selectedEntry_)
			tmpSurface = entry->selectedSurface;
		else
			tmpSurface = entry->unselectedSurface;

		rect.x = surface_-> w / 2 - tmpSurface->w / 2;
		rect.w = tmpSurface->w;
		rect.h = tmpSurface->h;
		if (isInside(x, y, rect))
		{
			selectedEntry_ = entry;
			return true;
		}
		rect.y += tmpSurface->h + SPACE_BETWEEN_ENTRIES;

	}
	return false;
}


/*
 * draws everything.
 */
void SdlMenu::drawMenu()
{
	drawBackground();
	drawEntries();
	if (!isDisplaySurface)
		SDL_BlitSurface(surface_, 0, SDL_GetVideoSurface(), 0);
	SDL_Flip(SDL_GetVideoSurface());
}



/*
 * returns true if (x, y) is inside the SDL_Rect
 */
bool SdlMenu::isInside(int x, int y, const SDL_Rect& rect)
{
	if (x > rect.x && x < rect.x + rect.w)
	{
		if (y > rect.y && y < rect.y + rect.h)
			return true;
	}

	return false;
}
