/***********************************************************************
 *
 *	VierVerliert
 *	(c) 2004, 2005 Thomas Unterthiner
 *
 * SdlUserInterface.cpp - interacts with the user
 *
 ************************************************************************/

#include <SDL.h>
#include <cassert>

#include "SdlUserInterface.h"
#include "SdlGridDrawer.h"
#include "GameManager.h"
#include "SdlMenu.h"
#include "SdlFontManager.h"
#include "Player.h"
#include "SdlHelper.h"



/*
 * Video-Settings
 */
const int XRES = 640;
const int YRES = 480;
const int BPP = 16;
const int VIDEOFLAGS = SDL_ANYFORMAT | SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_RLEACCEL;




/*
 * Ctor
 * in case SDL wasn't already initialized, start SDL and the video
 * in case we have initialized SDL, we are later also going to destroy
 * it in our Dtor
 */
SdlUserInterface::SdlUserInterface(GameManager& game)
	: UserInterface(game),
	  quitSdlOnExit_(false),
	  selectedSlot_(0)
{
	if (SDL_WasInit(SDL_INIT_VIDEO) == 0)
	{
		SDL_Init (SDL_INIT_VIDEO);
		display_ = SDL_SetVideoMode(XRES, YRES, BPP, VIDEOFLAGS);
		assert(display_);
		quitSdlOnExit_ = true;
	}
	else
		display_ = SDL_GetVideoSurface();

	SDL_WM_SetCaption("VierVerliert", 0);

	drawer_ = new SdlGridDrawer(game.getGrid(),
		game.getPlayer1(),game.getPlayer2(), display_);

	slotWidth_ = display_->w / getGame().getNrOfSlots();
	mainMenu_ = new SdlMenu(display_, EXIT_PROGRAM);
	mainMenu_->addEntry("start new game", START_GAME);
	mainMenu_->addEntry("quit program", EXIT_PROGRAM);
}



/*
 * Dtor
 */
SdlUserInterface::~SdlUserInterface()
{
	delete drawer_;
	if (quitSdlOnExit_)
	{
		SDL_FreeSurface(display_);
		SDL_Quit();
	}
	delete mainMenu_;
}

/*
 * check if there are new Events in the event-queue and deal with them
 * Note that this method will only return once the user has made any
 * input! (this maintains consistency with TextUserInterface!)
 *
 * The SDL-Cursor will be disabled, instead we draw our own cursor
 * with drawCursor() over the currently selected slot
 * NOTE that the SDL-Cursor will not be enabled when the function leaves!
 */
void SdlUserInterface::handleInput()
{
	SDL_Event event;
	block_ = true;

	SDL_ShowCursor(SDL_DISABLE);
	drawAll();

	while (block_)
	{
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
				case SDL_KEYUP:
					switch(event.key.keysym.sym)
					{
						case SDLK_ESCAPE:
							if (getGame().isRunning())
								getGame().exitToMainMenu();
							else
							    getGame().exitProgram();
							return;
						case SDLK_LEFT:
							if (selectedSlot_ > 0)
							    --selectedSlot_;
							else
							    selectedSlot_ = getGame().getNrOfSlots() - 1;
							break;
						case SDLK_RIGHT:
							++selectedSlot_;
							selectedSlot_ %= getGame().getNrOfSlots();
							break;
						case SDLK_RETURN:
							tryToUseSlot(selectedSlot_);
							break;
					}
					break;


				// note that the following two cases have a little problem:
				// as the GridDrawer determines internally where exactly the
				// slots lie, the event.button.x / slotWidth_ calculation
				// might fail in some cases!
				// (e.g. because event.button.x points outside of the grid-borders)
				// but it should work for most cases anyway ;)
				case SDL_MOUSEBUTTONUP:
					if (event.button.button == SDL_BUTTON_LEFT)
					{
						selectedSlot_ = event.button.x / slotWidth_;
						tryToUseSlot(selectedSlot_);
					}
					break;

				case SDL_MOUSEMOTION:
					selectedSlot_ = event.button.x / slotWidth_;
					if (selectedSlot_ >= getGame().getNrOfSlots())
						selectedSlot_ = getGame().getNrOfSlots() -1;
					break;

				case SDL_QUIT:
					getGame().exitProgram();

					// a break now won't do it, since it seems like
					// SDL pumps the event-queue full of SDL_QUIT events
					// once the SDL-window gets closed.
					// Therefore if we only use a break now SDL_PollEvent
					// ends up returning SDL_QUITs until eternity and we
					// never exit this function (let alone exit the
					// program as is our intent ;-) )
					return;
			}
			drawAll();
		}
		SDL_Delay(1); // don't eat to much CPU-Power ;-)
	}
}


/*
 * show who has won the game.
 * The message shown will bounce off the borders of the screen, and
 * the animation will end once the surface is flown out of the
 * upper left corner or if a button/key is pressed.
 * that's a stupid animation, but at least it's an animation ;-)
 *
 * NOTE: the screen won't be cleared before and while showing this 
 *       animation, so the animation will slowly draw over the old
 *       playing-scene
 */
void SdlUserInterface::showWinner(const Player& winner)
{
	SdlFont& font = SdlFontManager::get().getFont("Efont_Serif.ttf", 72);
	SDL_Surface* textSurface = 0;
	if (winner.code != NO_WINNER)
	{
		vector<SDL_Surface*> tmp;
		tmp.push_back(font.renderBlended("Congratulations!", 0, 0, 0));
		tmp.push_back(font.renderBlended(winner.name, winner.r, winner.g, winner.b));
		tmp.push_back(font.renderBlended("you have won!", 0, 0, 0));
		textSurface = blitOnOneSurface(tmp, ALIGN_CENTER);
	}
	else
	{
		textSurface = font.renderMultilineBlended(
			"BUNCH\nOF\nLOOSERS!\nnoone's won", 128, 128, 128, ALIGN_CENTER);
	}

	SDL_Rect pos;
	float x = 0.0;
	float y = 0.0;
	float dx = 0.5f;
	float dy = 0.5f;
	pos.x = pos.y = 0;

	SDL_BlitSurface(textSurface, 0, display_, &pos);
	SDL_Flip(display_);
	SDL_Delay(10); // wait long enough for the message to be read

	// clear all pending events
	SDL_Event event;
	SDL_Delay(100);
	while (SDL_PollEvent(0))
		SDL_PollEvent(&event);
	bool doLoop = true;
	while (doLoop)
	{
		while (SDL_PollEvent(&event))
		{
			switch (event.type)
			{
				case SDL_KEYUP:
				case SDL_MOUSEBUTTONUP:
					doLoop = false;
					break;

				case SDL_QUIT:
					doLoop = false;
					break;
			}
		}

		SDL_BlitSurface(textSurface, 0, display_, &pos);
		SDL_Flip(display_);
		SDL_Delay(10);

		if (pos.x + textSurface->w > display_->w) dx = -dx;
		if (pos.y + textSurface->h > display_->h) dy = -dy;

		x += dx;
		y += dy;
		pos.x = static_cast<Sint16>(x);
		pos.y = static_cast<Sint16>(y);
		if (pos.x + textSurface->w <= 0 || pos.y + textSurface->h <= 0) break;
	}
	SDL_FreeSurface(textSurface);
}


/*
 * try to insert a coin into the slot the user selected
 * if a valid slot got selected, we can disable the
 * block of handleInput();
 */
void SdlUserInterface::tryToUseSlot(unsigned short slot)
{
	try
	{
		getGame().putIntoSlot(slot);
	}
	catch(GameManager::IllegalSlotError)
	{
		handleInput();
	}

	block_ = false;
}


/*
 * the main menu: let the user do some stuff here
 */
void SdlUserInterface::showMainMenu()
{
	switch(mainMenu_->run())
	{
		case START_GAME:
			getGame().run();
			break;
		case EXIT_PROGRAM:
			getGame().exitProgram();
			break;
		default:
			// if we ever reach the following line, we've probably
			// added points to the mainmenu but forgotten to implement
			// 'em here ;-)
			assert(false);
			break;
	}
}



/*
 * draw everything
 */
void SdlUserInterface::drawAll()
{
	drawer_->setCursorToSlot(selectedSlot_);
	drawer_->draw();
	SDL_Flip(display_);
}
